unit fMAGAddEditDICOM;
{Package: MAG - VistA Imaging
WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
Date Created: 08/2009
Site Name: Silver Spring, OIFO
Developers: Bill Balshem
[==   unit fMAGAddEditDICOM;
 Description: User selects choices from the combo boxes to add a new record, edit record in the DICOM HEALTHCARE PROVIDER SERVICE configuration file
 for TELERETINAL IMAGING(2006.5831)
 The Procedure Index is a subset of the Specialty index and there will be cleared out whenever a specialty is selceted, the choices from the Procedure Index
 changes based on the Specialty Index selection.
==]
}
(*
      ;; +---------------------------------------------------------------------------------------------------+
      ;; Property of the US Government.
      ;; No permission to copy or redistribute this software is given.
      ;; Use of unreleased versions of this software requires the user
      ;;  to execute a written test agreement with the VistA Imaging
      ;;  Development Office of the Department of Veterans Affairs,
      ;;  telephone (301) 734-0100.
      ;;
      ;; The Food and Drug Administration classifies this software as
      ;; a medical device.  As such, it may not be changed
      ;; in any way.  Modifications to this software may result in an
      ;; adulterated medical device under 21CFR820, the use of which
      ;; is considered to be a violation of US Federal Statutes.
      ;; +---------------------------------------------------------------------------------------------------+

*)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, VA508AccessibilityManager;

type
  TfrmAddEditDICOM = class(TForm)
    lblSite: TLabel;
    lblName: TLabel;
    lblSpecialtyIndex: TLabel;
    lblInstruct: TLabel;
    cmbDivision: TComboBox;
    cmbServiceName: TComboBox;
    cmbServiceGroup: TComboBox;
    btnSave: TBitBtn;
    btnCancel: TBitBtn;
    imgServiceName: TImage;
    imgSpecialtyIndex: TImage;
    imgAcquisitionSite: TImage;
    Label1: TLabel;
    imgClinic: TImage;
    btnClinic: TButton;
    lstClinic: TListBox;
    amgrMain: TVA508AccessibilityManager;
    procedure SetSaveButton(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure PopulateComboBoxes;
    procedure SetSelections;
    procedure HelpClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnClinicClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure cmbDivisionChange(Sender: TObject);
  private
    { Private declarations }
    bChange: boolean;
    bChangeClinics: boolean;
    function BuildClinicIENStr: string;
  public
    { Public declarations }
    sIEN: string;
    sAlreadyAddedServices: string;
  end;

var
  frmAddEditDICOM: TfrmAddEditDICOM;

implementation

uses
  uMAGGlobalsTRA, dmMAGTeleReaderAdmin, MagFMComponents, trpcb, DiTypLib, fMAGSelectClinics;

{$R *.dfm}

procedure TfrmAddEditDICOM.SetSaveButton(Sender: TObject);
begin
  bChange := true;
  btnSave.Enabled := AllMandatoryFieldsSelected(Self);
end;

procedure TfrmAddEditDICOM.btnCancelClick(Sender: TObject);
begin
  if (not bChange) or (MessageDlg508(CancelSTR, mtConfirmation, [mbYes, mbNo], self.Handle)  = mrYes) then Close;
end;

procedure TfrmAddEditDICOM.PopulateComboBoxes;
begin
  PopulateComboBoxWithTextFilter(cmbServiceName, '123.5', '.01', sAlreadyAddedServices, ftRemove);
  PopulateComboBoxRPC(cmbServiceGroup, 'MAG4 INDEX GET SPECIALTY', emptystr, 4);
  PopComboWithLocalSites(cmbDivision, cmbServiceName.Enabled);
end;

procedure TfrmAddEditDICOM.SetSelections;
begin
  cmbServiceName.ItemIndex := cmbServiceName.Items.IndexOf(cmbServiceName.Text);
  cmbServiceGroup.ItemIndex := cmbServiceGroup.Items.IndexOf(cmbServiceGroup.Text);
  cmbDivision.ItemIndex := cmbDivision.Items.IndexOf(cmbDivision.Text);
end;

procedure TfrmAddEditDICOM.FormShow(Sender: TObject);
begin
  lblInstruct.Caption := sInstructions;
  PopulateComboBoxes;
  SetSelections;
  SetCombosReadOnly(Self);
  bChange := false;
  GetFormPosition(self);
end;

procedure TfrmAddEditDICOM.btnSaveClick(Sender: TObject);
var sResult: string; FormState: TFormState;
begin
  if cmbServiceName.Enabled then FormState := fsAdd else FormState := fsEdit;
  DataModule1.Broker.RemoteProcedure := 'MAG3 TELEREADER PDR SRVC SETUP';
  DataModule1.Broker.Param[0].PType := list;
  DataModule1.Broker.Param[0].Value := '.x';
  //IENs FROM Service Name Ref file, saved in combo box tstrings as TObject
  //after convert from string to integer - strings cant be cast as TObject
  //see combo box population code in GlobalsTRA unit
  DataModule1.Broker.Param[0].Mult['"REQUESTED SERVICE"'] := TIENObj(cmbServiceName.Items.Objects[cmbServiceName.ItemIndex]).sIEN;
  DataModule1.Broker.Param[0].Mult['"SERVICE GROUP"'] := TIENObj(cmbServiceGroup.Items.Objects[cmbServiceGroup.ItemIndex]).sIEN;
  DataModule1.Broker.Param[0].Mult['"SERVICE DIVISION"'] := TIENObj(cmbDivision.Items.Objects[cmbDivision.ItemIndex]).sIEN;
  if FormState = fsAdd then
    DataModule1.Broker.Param[0].Mult['"ACTION"'] := 'ADD'
  else if FormState = fsEdit then
    DataModule1.Broker.Param[0].Mult['"ACTION"'] := 'UPDATE';
  DataModule1.Broker.Param[0].Mult['"IEN"'] := sIEN;
  if ((FormState = fsAdd) and (lstClinic.Items.Count > 0)) or (bChangeClinics) then
    DataModule1.Broker.Param[0].Mult['"CLINIC"'] := BuildClinicIENStr;
  try
    sResult := DataModule1.Broker.strCall;
    MagLogger.Log('INFO', 'Successful ' + FormStateToStr(FormState) + ' to DICOM Healthcare Provider Service file.');
    ModalResult := mrOK;
  except
    on e: exception do
    begin
      MagLogger.Log('ERROR', e.Message);
      raise;
    end;
  end;
end;


procedure TfrmAddEditDICOM.HelpClick(Sender: TObject);
var s, sField: string;
begin
  case TImage(Sender).Tag of
    1: sField := '.01';
    else sField := IntToStr(TImage(Sender).Tag) 
  end;
  s := GetDataDictionaryHelp('2006.5831', sField);
  MessageDlg508(s, mtInformation, [mbOK], self.Handle);
end;

procedure TfrmAddEditDICOM.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = F1KEY then
  begin
    if ControlHasHelpText(ActiveControl) then
      begin
        if (ActiveControl.Name = 'cmbServiceName')  or (ActiveControl.Tag = 508) then HelpClick(imgServiceName)
        else if ActiveControl.Name = 'cmbServiceGroup' then HelpClick(imgSpecialtyIndex)
        else if ActiveControl.Name = 'cmbDivision' then HelpClick(imgAcquisitionSite)
        else if ActiveControl.Name = 'lstClinic' then HelpClick(imgClinic);
      end
    else
      begin
        OpenHelpFile;
      end
  end;
end;

procedure TfrmAddEditDICOM.btnClinicClick(Sender: TObject);
var frmSelectClinic: TfrmSelectClinic; 
begin
  if not JobCompleted then
  begin
    ShowMessage508('System is still retrieving clinics from database.' + #10 +#13 + 'Try back in 10-15 seconds.', self.Handle);
    exit;
  end;
  frmSelectClinic := TfrmSelectClinic.Create(nil);
  try
    frmSelectClinic.sSiteIEN := TIENObj(cmbDivision.Items.Objects[cmbDivision.ItemIndex]).sIEN;
    frmSelectClinic.lstSelected.Items := lstClinic.Items;
    if frmSelectClinic.ShowModal = mrOK then
    begin
      lstClinic.Items := frmSelectClinic.lstSelected.Items;
      bChangeClinics := true;
      SetSaveButton(self);
    end;
  finally
    frmSelectClinic.Free;
  end;
end;

function TfrmAddEditDICOM.BuildClinicIENStr: string;
var i: integer;
const DEL = '^';
begin
  result := '';
  for i := 0 to lstClinic.Items.Count - 1 do
    result := result + TIENObj(lstClinic.Items.Objects[i]).sIEN + DEL;
  result := copy(result, 1, length(result) - 1) //remove last DEL
end;

procedure TfrmAddEditDICOM.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SaveFormPosition(self);
end;

procedure TfrmAddEditDICOM.FormDestroy(Sender: TObject);
begin
  FreeStringsIENObj(cmbServiceName.Items);
  FreeStringsIENObj(cmbServiceGroup.Items);
  FreeStringsIENObj(cmbDivision.Items);
end;

procedure TfrmAddEditDICOM.cmbDivisionChange(Sender: TObject);
begin
  lstClinic.Items.Clear;
  bChangeClinics := true;
  SetSaveButton(Sender);
end;

end.
